import { createMemo, For, type Component, type JSX } from "solid-js";
import Scaffold from "~material/Scaffold";
import {
  AppBar,
  IconButton,
  List,
  ListItem,
  ListItemButton,
  ListItemSecondaryAction,
  ListItemText,
  ListSubheader,
  Radio,
  Switch,
  Toolbar,
} from "@suid/material";
import { ArrowBack } from "@suid/icons-material";
import iso639_1 from "iso-639-1";
import {
  autoMatchLangTag,
  createTranslator,
  SUPPORTED_LANGS,
} from "~platform/i18n";
import { Title } from "~material/typography";
import type { Template } from "@solid-primitives/i18n";
import { useStore } from "@nanostores/solid";
import { $settings } from "./stores";
import { useNavigator } from "~platform/StackedRouter";
import AppTopBar from "~material/AppTopBar";
import DocumentTitle from "~platform/DocumentTitle";

const ChooseLang: Component = () => {
  const { pop } = useNavigator();
  const [t] = createTranslator(
    () => import("./i18n/generic.json"),
    (code) =>
      import(`./i18n/${code}.json`) as Promise<{
        default: Record<string, string | undefined> & {
          ["lang.auto"]: Template<{ detected: string }>;
        };
      }>,
  );
  const settings = useStore($settings);

  const code = () => settings().language;

  const unsupportedLangCodes = createMemo(() => {
    return iso639_1.getAllCodes().filter((x) => !["zh", "en"].includes(x));
  });

  const matchedLangCode = createMemo(() => autoMatchLangTag());

  const onCodeChange = (code?: string) => {
    $settings.setKey("language", code);
  };

  return (
    <>
      <DocumentTitle>{t("Choose Language")}</DocumentTitle>
      <Scaffold
        topbar={
          <AppTopBar>
            <IconButton color="inherit" onClick={[pop, 1]} disableRipple>
              <ArrowBack />
            </IconButton>
            <Title>{t("Choose Language")}</Title>
          </AppTopBar>
        }
      >
        <List
          sx={{
            paddingBottom: "var(--safe-area-inset-bottom, 0)",
          }}
        >
          <ListItemButton
            onClick={() => {
              onCodeChange(code() ? undefined : matchedLangCode());
            }}
          >
            <ListItemText>
              {t("lang.auto", {
                detected: t(`lang.${matchedLangCode()}`) ?? matchedLangCode(),
              })}
            </ListItemText>
            <ListItemSecondaryAction>
              <Switch checked={typeof code() === "undefined"} />
            </ListItemSecondaryAction>
          </ListItemButton>
          <List subheader={<ListSubheader>{t("Supported")}</ListSubheader>}>
            <For each={SUPPORTED_LANGS}>
              {(c) => (
                <ListItemButton
                  disabled={typeof code() === "undefined"}
                  onClick={[onCodeChange, c]}
                >
                  <ListItemText>{t(`lang.${c}`)}</ListItemText>
                  <ListItemSecondaryAction>
                    <Radio
                      checked={
                        code() === c ||
                        (code() === undefined && matchedLangCode() == c)
                      }
                    />
                  </ListItemSecondaryAction>
                </ListItemButton>
              )}
            </For>
          </List>

          <List subheader={<ListSubheader>{t("Unsupported")}</ListSubheader>}>
            <For each={unsupportedLangCodes()}>
              {(code) => (
                <ListItem>
                  <ListItemText>{iso639_1.getNativeName(code)}</ListItemText>
                </ListItem>
              )}
            </For>
          </List>
        </List>
      </Scaffold>
    </>
  );
};

export default ChooseLang;
